try {
    $domain = [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain()
    $root = $domain.GetDirectoryEntry()
    $searcher = New-Object DirectoryServices.DirectorySearcher($root)
    $searcher.Filter = "(objectClass=user)"
    $searcher.PageSize = 1000

    $properties = @("samAccountName", "mail", "department", "company")
    foreach ($property in $properties) {
        $null = $searcher.PropertiesToLoad.Add($property)
    }

    $results = $searcher.FindAll()
    if ($results.Count -eq 0) {
        Write-Warning "No users found in the directory."
    }

    # Extract and filter user data
    $userData = foreach ($result in $results) {
        $user = $result.Properties

        $username = $user["samAccountName"] -join ''
        $email = $user["mail"] -join ''
        $department = $user["department"] -join ''
        $company = $user["company"] -join ''

        if (
            -not [string]::IsNullOrWhiteSpace($email) -and
            -not [string]::IsNullOrWhiteSpace($username) -and
            $username -notmatch '\$' -and
            $username -notmatch '^(SystemMailbox|HealthMailbox|DiscoverySearchMailbox|FederatedEmail|Migration\.)' -and
            $username -notin @("Administrator", "Guest", "krbtgt", "DefaultAccount")
        ) {
            [PSCustomObject]@{
                EmailAddress = $email
                Username     = $username
                Department   = $department
                Company      = $company
            }
        }
    }

    # Sort by Department (A–Z), then by Username
    $sorted = $userData | Sort-Object Department, Username

    $sorted | Format-Table EmailAddress, Username, Department, Company -AutoSize

} catch {
    Write-Error "An error occurred while processing the query: $_"
}
